#ifndef EXTRACONTROLSONWIDGETS_H
#define EXTRACONTROLSONWIDGETS_H

#include <iextracontrols.h>

#include <QMap>
#include <QCheckBox>
#include <QComboBox>
#include <QLabel>
#include <QGridLayout>

class QCheckBox;
class QComboBox;
class QLabel;

namespace LibFlyFileDialog
{

// keep this class header-only
class ExtraControlsOnWidgets : public IExtraControls
{
public:
    explicit ExtraControlsOnWidgets() {}
    ~ExtraControlsOnWidgets() {}

    bool supportsExtraControls() const override { return true; }

    void setControlEnabled(int id, bool on) override {
        auto it = m_extraControls.find(id);
        if (it != m_extraControls.end()) {
            (*it)->setEnabled(on);
        }
    }
    void setControlVisible(int id, bool on) override {
        auto it = m_extraControls.find(id);
        if (it != m_extraControls.end()) {
            (*it)->setVisible(on);
        }
    }
    void setControlText(int id, const QString &text) override
    {
        auto it = m_extraControls.find(id);
        if (it != m_extraControls.end()) {
            QWidget *w = *it;
            auto *checkBox = qobject_cast<QCheckBox *>(w);
            if (checkBox) {
                checkBox->setText(text);
            } else {
                auto *comboBox = qobject_cast<QComboBox *>(w);
                if (comboBox) {
                    auto *label = comboBox->itemData(0).value<QLabel *>();
                    if (label) {
                        label->setText(text);
                    }
                }
            }
        }
    }
    QString controlText(int id) const override
    {
        auto it = m_extraControls.find(id);
        if (it != m_extraControls.end()) {
            QWidget *w = *it;
            auto *checkBox = qobject_cast<QCheckBox *>(w);
            if (checkBox) {
                return checkBox->text();
            } else {
                auto *comboBox = qobject_cast<QComboBox *>(w);
                if (comboBox) {
                    auto *label = comboBox->itemData(0).value<QLabel *>();
                    if (label) {
                        return label->text();
                    }
                }
            }
        }
        return QString();
    }

    void addCheckBox(int id, const QString &text) override
    {
        auto it = m_extraControls.find(id);
        if (it != m_extraControls.end()) {
            delete *it;
            m_extraControls.erase(it);
        }

        auto *checkBox = new QCheckBox(text);
        addControl(id, checkBox);
        m_extraControls.insert(id, checkBox);
    }
    void setCheckBoxChecked(int id, bool on) override
    {
        auto *checkBox = qobject_cast<QCheckBox *>(m_extraControls.value(id));
        if (checkBox) {
            checkBox->setChecked(on);
        }
    }
    bool isCheckBoxChecked(int id) const override
    {
        auto *checkBox = qobject_cast<QCheckBox *>(m_extraControls.value(id));
        return checkBox ? checkBox->isChecked() : false;
    }

    void addComboBox(int id, const QString &text) override
    {
        auto it = m_extraControls.find(id);
        if (it != m_extraControls.end()) {
            delete *it;
            m_extraControls.erase(it);
        }

        auto *label = new QLabel(text);
        auto *comboBox = new QComboBox;
        comboBox->setItemData(0, QVariant::fromValue(label));
        label->setBuddy(comboBox);

        addControl(id, label, comboBox);

        m_extraControls.insert(id, comboBox);
    }
    void addComboBoxItem(int id, const QString &item) override
    {
        auto *comboBox = qobject_cast<QComboBox *>(m_extraControls.value(id));
        if (comboBox) {
            comboBox->addItem(item);
        }
    }
    void removeComboBoxItem(int id, int idx) override
    {
        auto *comboBox = qobject_cast<QComboBox *>(m_extraControls.value(id));
        if (comboBox) {
            comboBox->removeItem(idx);
        }
    }
    void setCurrentComboBoxItem(int id, int idx) override
    {
        auto *comboBox = qobject_cast<QComboBox *>(m_extraControls.value(id));
        if (comboBox) {
            comboBox->setCurrentIndex(idx);
        }
    }
    int currentComboBoxItem(int id) override
    {
        auto *comboBox = qobject_cast<QComboBox *>(m_extraControls.value(id));
        return comboBox ? comboBox->currentIndex() : -1;
    }

protected:
    virtual void addControl(int id, QCheckBox *checkBox) = 0;
    virtual void addControl(int id, QLabel *label, QComboBox *comboBox) = 0;

private:
    QMap<int, QWidget *> m_extraControls;
};

}

#endif
