#ifndef FLYFILEDIALOG_H
#define FLYFILEDIALOG_H

#include <QObject>

#include "ifiledialog.h"
#include "flyfiledialog_global.h"

namespace LibFlyFileDialog
{
class AutoSelectExtension;
}

class FLY_FILEDIALOG_EXPORT FlyFileDialog : public QObject, public LibFlyFileDialog::IFileDialog
{
    Q_OBJECT
public:
    explicit FlyFileDialog(QWidget *parent = nullptr,
                           const QString &caption = QString(),
                           const QString &directory = QString(),
                           const QString &filter = QString());

    ~FlyFileDialog();

    void setDirectory(const QString &directory) override;
    QString directory() const override;

    void selectFile(const QString &filename) override;
    QStringList selectedFiles() const override;

    void setNameFilter(const QString &filter) override;
    void setNameFilters(const QStringList& filters) override;
    void selectNameFilter(const QString &filter) override;
    QString selectedNameFilter() const override;

    QDir::Filters filter() const override;
    void setFilter(QDir::Filters filters) override;

    void setViewMode(QFileDialog::ViewMode mode) override;
    QFileDialog::ViewMode viewMode() const override;

    void setFileMode(QFileDialog::FileMode mode) override;
    QFileDialog::FileMode fileMode() const override;

    void setAcceptMode(QFileDialog::AcceptMode mode) override;
    QFileDialog::AcceptMode acceptMode() const override;

    void setConfirmOverwrite(bool enabled) override;
    bool confirmOverwrite() const override;

    void setDefaultSuffix(const QString &suffix) override;
    QString defaultSuffix() const override;

    void setSidebarUrls(const QList<QUrl> & urls) override;
    QList<QUrl> sidebarUrls() const override;

    void setVisible(bool visible) override;

    void setShowOptions(Qt::WindowFlags windowFlags,
                        Qt::WindowModality windowModality,
                        QWindow *parent) override;

    static QString getOpenFileName(QWidget *parent = nullptr,
                                   const QString &caption = QString(),
                                   const QString &dir = QString(),
                                   const QString &filter = QString(),
                                   QString *selectedFilter = nullptr);

    static QString getSaveFileName(QWidget *parent = nullptr,
                                   const QString &caption = QString(),
                                   const QString &dir = QString(),
                                   const QString &filter = QString(),
                                   QString *selectedFilter = nullptr);

    static QString getExistingDirectory(QWidget *parent = nullptr,
                                        const QString &caption = QString(),
                                        const QString &dir = QString());

    static QStringList getOpenFileNames(QWidget *parent = nullptr,
                                        const QString &caption = QString(),
                                        const QString &dir = QString(),
                                        const QString &filter = QString(),
                                        QString *selectedFilter = nullptr);

    static bool isExternal();

    QString windowTitle() const override;

    QObject *signalsObject() const override;

public Q_SLOTS:
    void setWindowTitle(const QString &) override;
    int exec() override;

Q_SIGNALS:
    void currentUrlChanged(const QUrl & url);
    void directoryUrlEntered(const QUrl & directory);
    void urlSelected(const QUrl & url);
    void urlsSelected(const QList<QUrl> & urls);
    void filterSelected(const QString & filter);
    void accepted();
    void rejected();

private:
    static LibFlyFileDialog::IFileDialog* createFileDialog(QWidget *parent = nullptr);
    LibFlyFileDialog::IFileDialog* fileDialog;
    LibFlyFileDialog::AutoSelectExtension *autoSelectExtension;
};

#endif
